@@ -9,7 +9,7 @@ use hickory_resolver::proto::{
         DNSClass, RData, Record, RecordType,
     },
 };
-use log::{debug, trace};
+use log::{debug, trace, warn};
 
 use super::manager::FakeDnsManager;
 
@@ -26,30 +26,37 @@ pub async fn handle_dns_request(req_message: &Message, manager: &FakeDnsManager)
             rsp_message.add_query(query.clone());
 
             if query.query_class() != DNSClass::IN {
-                let record = Record::<RData>::with(query.name().clone(), query.query_type(), 0);
-                rsp_message.add_answer(record);
+                // let record = Record::<RData>::from_rdata(query.name().clone(), 0, query.query_type());
+                // rsp_message.add_answer(record);
+                warn!(
+                    "Query class: {:?} is not supported. Full {:?}",
+                    query.query_class(),
+                    req_message
+                );
                 continue;
             }
 
             match query.query_type() {
                 RecordType::A => {
                     let (ip_addr, expire_duration) = manager.map_domain_ipv4(query.name()).await?;
-                    let [a, b, c, d] = ip_addr.octets();
 
-                    let mut record =
-                        Record::<RData>::with(query.name().clone(), RecordType::A, expire_duration.as_secs() as u32);
+                    let mut record = Record::<RData>::from_rdata(
+                        query.name().clone(),
+                        expire_duration.as_secs() as u32,
+                        RData::A(A(ip_addr)),
+                    );
                     record.set_dns_class(query.query_class());
-                    record.set_data(Some(RData::A(A::new(a, b, c, d))));
                     rsp_message.add_answer(record);
                 }
                 RecordType::AAAA => {
                     let (ip_addr, expire_duration) = manager.map_domain_ipv6(query.name()).await?;
-                    let [a, b, c, d, e, f, g, h] = ip_addr.segments();
 
-                    let mut record =
-                        Record::<RData>::with(query.name().clone(), RecordType::AAAA, expire_duration.as_secs() as u32);
+                    let mut record = Record::<RData>::from_rdata(
+                        query.name().clone(),
+                        expire_duration.as_secs() as u32,
+                        RData::AAAA(AAAA(ip_addr)),
+                    );
                     record.set_dns_class(query.query_class());
-                    record.set_data(Some(RData::AAAA(AAAA::new(a, b, c, d, e, f, g, h))));
                     rsp_message.add_answer(record);
                 }
                 _ => {
