@@ -1,27 +1,25 @@
 //! Shadowsocks Local Server
 
 use std::{
-    future::Future,
     io::{self, ErrorKind},
-    pin::Pin,
+    net::SocketAddr,
     sync::Arc,
-    task::{Context, Poll},
     time::Duration,
 };
 
-use futures::{future, ready};
+use futures::future;
 use log::trace;
 use shadowsocks::{
     config::Mode,
     net::{AcceptOpts, ConnectOpts},
 };
-use tokio::task::JoinHandle;
 
 #[cfg(feature = "local-flow-stat")]
 use crate::{config::LocalFlowStatAddress, net::FlowStat};
 use crate::{
     config::{Config, ConfigType, ProtocolType},
     dns::build_dns_resolver,
+    utils::ServerHandle,
 };
 
 use self::{
@@ -70,27 +68,6 @@ pub mod utils;
 /// This is borrowed from Go's `net` library's default setting
 pub(crate) const LOCAL_DEFAULT_KEEPALIVE_TIMEOUT: Duration = Duration::from_secs(15);
 
-struct ServerHandle(JoinHandle<io::Result<()>>);
-
-impl Drop for ServerHandle {
-    #[inline]
-    fn drop(&mut self) {
-        self.0.abort();
-    }
-}
-
-impl Future for ServerHandle {
-    type Output = io::Result<()>;
-
-    #[inline]
-    fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        match ready!(Pin::new(&mut self.0).poll(cx)) {
-            Ok(res) => res.into(),
-            Err(err) => Err(io::Error::new(ErrorKind::Other, err)).into(),
-        }
-    }
-}
-
 /// Local Server instance
 pub struct Server {
     balancer: PingBalancer,
@@ -154,7 +131,7 @@ impl Server {
             vpn_protect_path: config.outbound_vpn_protect_path,
 
             bind_interface: config.outbound_bind_interface,
-            bind_local_addr: config.outbound_bind_addr,
+            bind_local_addr: config.outbound_bind_addr.map(|ip| SocketAddr::new(ip, 0)),
 
             ..Default::default()
         };
@@ -165,6 +142,7 @@ impl Server {
         connect_opts.tcp.keepalive = config.keep_alive.or(Some(LOCAL_DEFAULT_KEEPALIVE_TIMEOUT));
         connect_opts.tcp.mptcp = config.mptcp;
         connect_opts.udp.mtu = config.udp_mtu;
+        connect_opts.udp.allow_fragmentation = config.outbound_udp_allow_fragmentation;
         context.set_connect_opts(connect_opts);
 
         let mut accept_opts = AcceptOpts {
@@ -310,6 +288,9 @@ impl Server {
                     if let Some(b) = local_config.udp_addr {
                         server_builder.set_udp_bind_addr(b.clone());
                     }
+                    if let Some(b) = local_config.udp_associate_addr {
+                        server_builder.set_udp_associate_addr(b.clone());
+                    }
 
                     #[cfg(target_os = "macos")]
                     if let Some(n) = local_config.launchd_tcp_socket_name {
