@@ -5,7 +5,6 @@ use std::{
     os::unix::io::AsRawFd,
 };
 
-use async_trait::async_trait;
 use log::warn;
 use shadowsocks::net::{is_dual_stack_addr, set_tcp_fastopen, AcceptOpts};
 use socket2::SockAddr;
@@ -19,7 +18,6 @@ use crate::{
     },
 };
 
-#[async_trait]
 impl TcpListenerRedirExt for TcpListener {
     async fn bind_redir(ty: RedirType, addr: SocketAddr, accept_opts: AcceptOpts) -> io::Result<TcpListener> {
         match ty {
