@@ -7,7 +7,7 @@ use std::{
     time::Duration,
 };
 
-use futures::{stream::FuturesUnordered, FutureExt, StreamExt};
+use futures::future;
 use log::{error, trace};
 use shadowsocks::{
     config::{ManagerAddr, ServerConfig},
@@ -18,13 +18,13 @@ use shadowsocks::{
 };
 use tokio::time;
 
-use crate::{acl::AccessControl, config::SecurityConfig, net::FlowStat};
+use crate::{acl::AccessControl, config::SecurityConfig, net::FlowStat, utils::ServerHandle};
 
 use super::{context::ServiceContext, tcprelay::TcpServer, udprelay::UdpServer};
 
 /// Shadowsocks Server Builder
 pub struct ServerBuilder {
-    context: Arc<ServiceContext>,
+    context: ServiceContext,
     svr_cfg: ServerConfig,
     udp_expiry_duration: Option<Duration>,
     udp_capacity: Option<usize>,
@@ -35,11 +35,11 @@ pub struct ServerBuilder {
 impl ServerBuilder {
     /// Create a new server builder from configuration
     pub fn new(svr_cfg: ServerConfig) -> ServerBuilder {
-        ServerBuilder::with_context(Arc::new(ServiceContext::new()), svr_cfg)
+        ServerBuilder::with_context(ServiceContext::new(), svr_cfg)
     }
 
     /// Create a new server builder with context
-    pub fn with_context(context: Arc<ServiceContext>, svr_cfg: ServerConfig) -> ServerBuilder {
+    fn with_context(context: ServiceContext, svr_cfg: ServerConfig) -> ServerBuilder {
         ServerBuilder {
             context,
             svr_cfg,
@@ -62,8 +62,7 @@ impl ServerBuilder {
 
     /// Set `ConnectOpts`
     pub fn set_connect_opts(&mut self, opts: ConnectOpts) {
-        let context = Arc::get_mut(&mut self.context).expect("cannot set ConnectOpts on a shared context");
-        context.set_connect_opts(opts)
+        self.context.set_connect_opts(opts)
     }
 
     /// Set UDP association's expiry duration
@@ -88,14 +87,12 @@ impl ServerBuilder {
 
     /// Set customized DNS resolver
     pub fn set_dns_resolver(&mut self, resolver: Arc<DnsResolver>) {
-        let context = Arc::get_mut(&mut self.context).expect("cannot set DNS resolver on a shared context");
-        context.set_dns_resolver(resolver)
+        self.context.set_dns_resolver(resolver)
     }
 
     /// Set access control list
     pub fn set_acl(&mut self, acl: Arc<AccessControl>) {
-        let context = Arc::get_mut(&mut self.context).expect("cannot set ACL on a shared context");
-        context.set_acl(acl);
+        self.context.set_acl(acl);
     }
 
     /// Set `AcceptOpts` for accepting new connections
@@ -105,14 +102,12 @@ impl ServerBuilder {
 
     /// Try to connect IPv6 addresses first if hostname could be resolved to both IPv4 and IPv6
     pub fn set_ipv6_first(&mut self, ipv6_first: bool) {
-        let context = Arc::get_mut(&mut self.context).expect("cannot set ipv6_first on a shared context");
-        context.set_ipv6_first(ipv6_first);
+        self.context.set_ipv6_first(ipv6_first);
     }
 
     /// Set security config
     pub fn set_security_config(&mut self, security: &SecurityConfig) {
-        let context = Arc::get_mut(&mut self.context).expect("cannot set security on a shared context");
-        context.set_security_config(security)
+        self.context.set_security_config(security)
     }
 
     /// Start the server
@@ -121,6 +116,8 @@ impl ServerBuilder {
     /// 2. Starts TCP server (listener)
     /// 3. Starts UDP server (listener)
     pub async fn build(mut self) -> io::Result<Server> {
+        let context = Arc::new(self.context);
+
         let mut plugin = None;
 
         if let Some(plugin_cfg) = self.svr_cfg.plugin() {
@@ -131,14 +128,14 @@ impl ServerBuilder {
 
         let mut tcp_server = None;
         if self.svr_cfg.mode().enable_tcp() {
-            let server = TcpServer::new(self.context.clone(), self.svr_cfg.clone(), self.accept_opts.clone()).await?;
+            let server = TcpServer::new(context.clone(), self.svr_cfg.clone(), self.accept_opts.clone()).await?;
             tcp_server = Some(server);
         }
 
         let mut udp_server = None;
         if self.svr_cfg.mode().enable_udp() {
             let server = UdpServer::new(
-                self.context.clone(),
+                context.clone(),
                 self.svr_cfg.clone(),
                 self.udp_expiry_duration,
                 self.udp_capacity,
@@ -149,7 +146,7 @@ impl ServerBuilder {
         }
 
         Ok(Server {
-            context: self.context,
+            context,
             svr_cfg: self.svr_cfg,
             tcp_server,
             udp_server,
@@ -187,36 +184,33 @@ impl Server {
 
     /// Start serving
     pub async fn run(self) -> io::Result<()> {
-        let vfut = FuturesUnordered::new();
+        let mut vfut = Vec::new();
 
         if let Some(plugin) = self.plugin {
-            vfut.push(
-                async move {
-                    match plugin.join().await {
-                        Ok(status) => {
-                            error!("plugin exited with status: {}", status);
-                            Ok(())
-                        }
-                        Err(err) => {
-                            error!("plugin exited with error: {}", err);
-                            Err(err)
-                        }
+            vfut.push(ServerHandle(tokio::spawn(async move {
+                match plugin.join().await {
+                    Ok(status) => {
+                        error!("plugin exited with status: {}", status);
+                        Ok(())
+                    }
+                    Err(err) => {
+                        error!("plugin exited with error: {}", err);
+                        Err(err)
                     }
                 }
-                .boxed(),
-            );
+            })));
         }
 
         if let Some(tcp_server) = self.tcp_server {
-            vfut.push(tcp_server.run().boxed());
+            vfut.push(ServerHandle(tokio::spawn(tcp_server.run())));
         }
 
         if let Some(udp_server) = self.udp_server {
-            vfut.push(udp_server.run().boxed())
+            vfut.push(ServerHandle(tokio::spawn(udp_server.run())));
         }
 
         if let Some(manager_addr) = self.manager_addr {
-            let manager_fut = async move {
+            vfut.push(ServerHandle(tokio::spawn(async move {
                 loop {
                     match ManagerClient::connect(
                         self.context.context_ref(),
@@ -251,13 +245,10 @@ impl Server {
                     // Report every 10 seconds
                     time::sleep(Duration::from_secs(10)).await;
                 }
-            }
-            .boxed();
-            vfut.push(manager_fut);
+            })));
         }
 
-        let (res, _) = vfut.into_future().await;
-        if let Some(Err(err)) = res {
+        if let (Err(err), ..) = future::select_all(vfut).await {
             error!("servers exited with error: {}", err);
         }
 
