@@ -5,7 +5,7 @@ use std::net::SocketAddr;
 #[cfg(unix)]
 pub use self::sys::uds::{UnixListener, UnixStream};
 pub use self::{
-    option::{AcceptOpts, ConnectOpts, TcpSocketOpts},
+    option::{AcceptOpts, ConnectOpts, TcpSocketOpts, UdpSocketOpts},
     sys::{get_ip_stack_capabilities, set_tcp_fastopen, socket_bind_dual_stack, IpStackCapabilities},
     tcp::{TcpListener, TcpStream},
     udp::UdpSocket,
