@@ -1,5 +1,9 @@
 //! UDP socket for communicating with shadowsocks' proxy server
 
+#[cfg(unix)]
+use std::os::fd::{AsFd, AsRawFd, BorrowedFd, IntoRawFd, RawFd};
+#[cfg(windows)]
+use std::os::windows::io::{AsRawSocket, AsSocket, BorrowedSocket, IntoRawSocket, RawSocket};
 use std::{
     io::{self, ErrorKind},
     net::SocketAddr,
@@ -12,7 +16,7 @@ use byte_string::ByteStr;
 use bytes::{Bytes, BytesMut};
 use log::{info, trace, warn};
 use once_cell::sync::Lazy;
-use tokio::{io::ReadBuf, net::ToSocketAddrs, time};
+use tokio::{io::ReadBuf, time};
 
 use crate::{
     config::{ServerAddr, ServerConfig, ServerUserManager},
@@ -22,9 +26,12 @@ use crate::{
     relay::{socks5::Address, udprelay::options::UdpSocketControlData},
 };
 
-use super::crypto_io::{
-    decrypt_client_payload, decrypt_server_payload, encrypt_client_payload, encrypt_server_payload, ProtocolError,
-    ProtocolResult,
+use super::{
+    compat::{DatagramReceive, DatagramReceiveExt, DatagramSend, DatagramSendExt, DatagramSocket},
+    crypto_io::{
+        decrypt_client_payload, decrypt_server_payload, encrypt_client_payload, encrypt_server_payload, ProtocolError,
+        ProtocolResult,
+    },
 };
 
 static DEFAULT_CONNECT_OPTS: Lazy<ConnectOpts> = Lazy::new(Default::default);
@@ -66,9 +73,10 @@ impl From<ProxySocketError> for io::Error {
 pub type ProxySocketResult<T> = Result<T, ProxySocketError>;
 
 /// UDP client for communicating with ShadowSocks' server
-pub struct ProxySocket {
+#[derive(Debug)]
+pub struct ProxySocket<S> {
     socket_type: UdpSocketType,
-    socket: ShadowUdpSocket,
+    io: S,
     method: CipherKind,
     key: Box<[u8]>,
     send_timeout: Option<Duration>,
@@ -78,9 +86,12 @@ pub struct ProxySocket {
     user_manager: Option<Arc<ServerUserManager>>,
 }
 
-impl ProxySocket {
+impl ProxySocket<ShadowUdpSocket> {
     /// Create a client to communicate with Shadowsocks' UDP server (outbound)
-    pub async fn connect(context: SharedContext, svr_cfg: &ServerConfig) -> ProxySocketResult<ProxySocket> {
+    pub async fn connect(
+        context: SharedContext,
+        svr_cfg: &ServerConfig,
+    ) -> ProxySocketResult<ProxySocket<ShadowUdpSocket>> {
         ProxySocket::connect_with_opts(context, svr_cfg, &DEFAULT_CONNECT_OPTS)
             .await
             .map_err(Into::into)
@@ -91,7 +102,7 @@ impl ProxySocket {
         context: SharedContext,
         svr_cfg: &ServerConfig,
         opts: &ConnectOpts,
-    ) -> ProxySocketResult<ProxySocket> {
+    ) -> ProxySocketResult<ProxySocket<ShadowUdpSocket>> {
         // Note: Plugins doesn't support UDP relay
 
         let socket = ShadowUdpSocket::connect_server_with_opts(&context, svr_cfg.udp_external_addr(), opts).await?;
@@ -111,42 +122,11 @@ impl ProxySocket {
         ))
     }
 
-    /// Create a `ProxySocket` from a `UdpSocket`
-    pub fn from_socket<S>(
-        socket_type: UdpSocketType,
+    /// Create a `ProxySocket` binding to a specific address (inbound)
+    pub async fn bind(
         context: SharedContext,
         svr_cfg: &ServerConfig,
-        socket: S,
-    ) -> ProxySocket
-    where
-        S: Into<ShadowUdpSocket>,
-    {
-        let key = svr_cfg.key().to_vec().into_boxed_slice();
-        let method = svr_cfg.method();
-
-        // NOTE: svr_cfg.timeout() is not for this socket, but for associations.
-
-        ProxySocket {
-            socket_type,
-            socket: socket.into(),
-            method,
-            key,
-            send_timeout: None,
-            recv_timeout: None,
-            context,
-            identity_keys: match socket_type {
-                UdpSocketType::Client => svr_cfg.clone_identity_keys(),
-                UdpSocketType::Server => Arc::new(Vec::new()),
-            },
-            user_manager: match socket_type {
-                UdpSocketType::Client => None,
-                UdpSocketType::Server => svr_cfg.clone_user_manager(),
-            },
-        }
-    }
-
-    /// Create a `ProxySocket` binding to a specific address (inbound)
-    pub async fn bind(context: SharedContext, svr_cfg: &ServerConfig) -> ProxySocketResult<ProxySocket> {
+    ) -> ProxySocketResult<ProxySocket<ShadowUdpSocket>> {
         ProxySocket::bind_with_opts(context, svr_cfg, AcceptOpts::default())
             .await
             .map_err(Into::into)
@@ -157,7 +137,7 @@ impl ProxySocket {
         context: SharedContext,
         svr_cfg: &ServerConfig,
         opts: AcceptOpts,
-    ) -> ProxySocketResult<ProxySocket> {
+    ) -> ProxySocketResult<ProxySocket<ShadowUdpSocket>> {
         // Plugins doesn't support UDP
         let socket = match svr_cfg.udp_external_addr() {
             ServerAddr::SocketAddr(sa) => ShadowUdpSocket::listen_with_opts(sa, opts).await?,
@@ -175,7 +155,54 @@ impl ProxySocket {
             socket,
         ))
     }
+}
 
+impl<S> ProxySocket<S> {
+    /// Create a `ProxySocket` from a I/O object that impls `DatagramTransport`
+    pub fn from_socket(
+        socket_type: UdpSocketType,
+        context: SharedContext,
+        svr_cfg: &ServerConfig,
+        socket: S,
+    ) -> ProxySocket<S> {
+        let key = svr_cfg.key().to_vec().into_boxed_slice();
+        let method = svr_cfg.method();
+
+        // NOTE: svr_cfg.timeout() is not for this socket, but for associations.
+        ProxySocket {
+            socket_type,
+            io: socket,
+            method,
+            key,
+            send_timeout: None,
+            recv_timeout: None,
+            context,
+            identity_keys: match socket_type {
+                UdpSocketType::Client => svr_cfg.clone_identity_keys(),
+                UdpSocketType::Server => Arc::new(Vec::new()),
+            },
+            user_manager: match socket_type {
+                UdpSocketType::Client => None,
+                UdpSocketType::Server => svr_cfg.clone_user_manager(),
+            },
+        }
+    }
+
+    /// Set `send` timeout, `None` will clear timeout
+    pub fn set_send_timeout(&mut self, t: Option<Duration>) {
+        self.send_timeout = t;
+    }
+
+    /// Set `recv` timeout, `None` will clear timeout
+    pub fn set_recv_timeout(&mut self, t: Option<Duration>) {
+        self.recv_timeout = t;
+    }
+}
+
+impl<S> ProxySocket<S>
+where
+    S: DatagramSend,
+{
     fn encrypt_send_buffer(
         &self,
         addr: &Address,
@@ -237,8 +264,8 @@ impl ProxySocket {
         );
 
         let send_len = match self.send_timeout {
-            None => self.socket.send(&send_buf).await?,
-            Some(d) => match time::timeout(d, self.socket.send(&send_buf)).await {
+            None => self.io.send(&send_buf).await?,
+            Some(d) => match time::timeout(d, self.io.send(&send_buf)).await {
                 Ok(Ok(l)) => l,
                 Ok(Err(err)) => return Err(err.into()),
                 Err(..) => return Err(io::Error::from(ErrorKind::TimedOut).into()),
@@ -291,7 +318,7 @@ impl ProxySocket {
 
         let n_send_buf = send_buf.len();
 
-        match self.socket.poll_send(cx, &send_buf).map_err(|x| x.into()) {
+        match self.io.poll_send(cx, &send_buf).map_err(|x| x.into()) {
             Poll::Ready(Ok(l)) => {
                 if l == n_send_buf {
                     Poll::Ready(Ok(payload.len()))
@@ -336,14 +363,14 @@ impl ProxySocket {
         self.encrypt_send_buffer(addr, control, &self.identity_keys, payload, &mut send_buf)?;
 
         info!(
-            "UDP server client send to {}, payload length {} bytes, packet length {} bytes",
+            "UDP server client poll_send_to to {}, payload length {} bytes, packet length {} bytes",
             target,
             payload.len(),
             send_buf.len()
         );
 
         let n_send_buf = send_buf.len();
-        match self.socket.poll_send_to(cx, &send_buf, target).map_err(|x| x.into()) {
+        match self.io.poll_send_to(cx, &send_buf, target).map_err(|x| x.into()) {
             Poll::Ready(Ok(l)) => {
                 if l == n_send_buf {
                     Poll::Ready(Ok(payload.len()))
@@ -359,25 +386,20 @@ impl ProxySocket {
     ///
     /// Check if socket is ready to `send`, or writable.
     pub fn poll_send_ready(&self, cx: &mut Context<'_>) -> Poll<ProxySocketResult<()>> {
-        self.socket.poll_send_ready(cx).map_err(|x| x.into())
+        self.io.poll_send_ready(cx).map_err(|x| x.into())
     }
 
     /// Send a UDP packet to target through proxy `target`
-    pub async fn send_to<A: ToSocketAddrs>(
-        &self,
-        target: A,
-        addr: &Address,
-        payload: &[u8],
-    ) -> ProxySocketResult<usize> {
+    pub async fn send_to(&self, target: SocketAddr, addr: &Address, payload: &[u8]) -> ProxySocketResult<usize> {
         self.send_to_with_ctrl(target, addr, &DEFAULT_SOCKET_CONTROL, payload)
             .await
             .map_err(Into::into)
     }
 
     /// Send a UDP packet to target through proxy `target`
-    pub async fn send_to_with_ctrl<A: ToSocketAddrs>(
+    pub async fn send_to_with_ctrl(
         &self,
-        target: A,
+        target: SocketAddr,
         addr: &Address,
         control: &UdpSocketControlData,
         payload: &[u8],
@@ -386,7 +408,7 @@ impl ProxySocket {
         self.encrypt_send_buffer(addr, control, &self.identity_keys, payload, &mut send_buf)?;
 
         trace!(
-            "UDP server client send to, addr {}, control: {:?}, payload length {} bytes, packet length {} bytes",
+            "UDP server client send_to to, addr {}, control: {:?}, payload length {} bytes, packet length {} bytes",
             addr,
             control,
             payload.len(),
@@ -394,8 +416,8 @@ impl ProxySocket {
         );
 
         let send_len = match self.send_timeout {
-            None => self.socket.send_to(&send_buf, target).await?,
-            Some(d) => match time::timeout(d, self.socket.send_to(&send_buf, target)).await {
+            None => self.io.send_to(&send_buf, target).await?,
+            Some(d) => match time::timeout(d, self.io.send_to(&send_buf, target)).await {
                 Ok(Ok(l)) => l,
                 Ok(Err(err)) => return Err(err.into()),
                 Err(..) => return Err(io::Error::from(ErrorKind::TimedOut).into()),
@@ -404,7 +426,7 @@ impl ProxySocket {
 
         if send_buf.len() != send_len {
             warn!(
-                "UDP server client send {} bytes, but actually sent {} bytes",
+                "UDP server client send_to {} bytes, but actually sent {} bytes",
                 send_buf.len(),
                 send_len
             );
@@ -412,7 +434,12 @@ impl ProxySocket {
 
         Ok(send_len)
     }
+}
 
+impl<S> ProxySocket<S>
+where
+    S: DatagramReceive,
+{
     fn decrypt_recv_buffer(
         &self,
         recv_buf: &mut [u8],
@@ -444,10 +471,9 @@ impl ProxySocket {
         &self,
         recv_buf: &mut [u8],
     ) -> ProxySocketResult<(usize, Address, usize, Option<UdpSocketControlData>)> {
-        // Waiting for response from server SERVER -> CLIENT
         let recv_n = match self.recv_timeout {
-            None => self.socket.recv(recv_buf).await?,
-            Some(d) => match time::timeout(d, self.socket.recv(recv_buf)).await {
+            None => self.io.recv(recv_buf).await?,
+            Some(d) => match time::timeout(d, self.io.recv(recv_buf)).await {
                 Ok(Ok(l)) => l,
                 Ok(Err(err)) => return Err(err.into()),
                 Err(..) => return Err(io::Error::from(ErrorKind::TimedOut).into()),
@@ -494,8 +520,8 @@ impl ProxySocket {
     ) -> ProxySocketResult<(usize, SocketAddr, Address, usize, Option<UdpSocketControlData>)> {
         // Waiting for response from server SERVER -> CLIENT
         let (recv_n, target_addr) = match self.recv_timeout {
-            None => self.socket.recv_from(recv_buf).await?,
-            Some(d) => match time::timeout(d, self.socket.recv_from(recv_buf)).await {
+            None => self.io.recv_from(recv_buf).await?,
+            Some(d) => match time::timeout(d, self.io.recv_from(recv_buf)).await {
                 Ok(Ok(l)) => l,
                 Ok(Err(err)) => return Err(err.into()),
                 Err(..) => return Err(io::Error::from(ErrorKind::TimedOut).into()),
@@ -538,7 +564,7 @@ impl ProxySocket {
         cx: &mut Context<'_>,
         recv_buf: &mut ReadBuf,
     ) -> Poll<ProxySocketResult<(usize, Address, usize, Option<UdpSocketControlData>)>> {
-        ready!(self.socket.poll_recv(cx, recv_buf))?;
+        ready!(self.io.poll_recv(cx, recv_buf))?;
 
         let n_recv = recv_buf.filled().len();
 
@@ -566,7 +592,7 @@ impl ProxySocket {
         cx: &mut Context<'_>,
         recv_buf: &mut ReadBuf,
     ) -> Poll<ProxySocketResult<(usize, SocketAddr, Address, usize, Option<UdpSocketControlData>)>> {
-        let src = ready!(self.socket.poll_recv_from(cx, recv_buf))?;
+        let src = ready!(self.io.poll_recv_from(cx, recv_buf))?;
 
         let n_recv = recv_buf.filled().len();
         match self.decrypt_recv_buffer(recv_buf.filled_mut(), self.user_manager.as_deref()) {
@@ -577,21 +603,77 @@ impl ProxySocket {
 
     /// poll family functions
     pub fn poll_recv_ready(&self, cx: &mut Context<'_>) -> Poll<ProxySocketResult<()>> {
-        self.socket.poll_recv_ready(cx).map_err(|x| x.into())
+        self.io.poll_recv_ready(cx).map_err(|x| x.into())
     }
+}
 
+impl<S> ProxySocket<S>
+where
+    S: DatagramSocket,
+{
     /// Get local addr of socket
     pub fn local_addr(&self) -> io::Result<SocketAddr> {
-        self.socket.local_addr()
+        self.io.local_addr()
     }
+}
 
-    /// Set `send` timeout, `None` will clear timeout
-    pub fn set_send_timeout(&mut self, t: Option<Duration>) {
-        self.send_timeout = t;
+#[cfg(unix)]
+impl<S> AsRawFd for ProxySocket<S>
+where
+    S: AsRawFd,
+{
+    /// Retrieve raw fd of the outbound socket
+    fn as_raw_fd(&self) -> RawFd {
+        self.io.as_raw_fd()
     }
+}
 
-    /// Set `recv` timeout, `None` will clear timeout
-    pub fn set_recv_timeout(&mut self, t: Option<Duration>) {
-        self.recv_timeout = t;
+#[cfg(unix)]
+impl<S> AsFd for ProxySocket<S>
+where
+    S: AsFd,
+{
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.io.as_fd()
+    }
+}
+
+#[cfg(unix)]
+impl<S> IntoRawFd for ProxySocket<S>
+where
+    S: IntoRawFd,
+{
+    fn into_raw_fd(self) -> RawFd {
+        self.io.into_raw_fd()
+    }
+}
+
+#[cfg(windows)]
+impl<S> AsRawSocket for ProxySocket<S>
+where
+    S: AsRawSocket,
+{
+    fn as_raw_socket(&self) -> RawSocket {
+        self.io.as_raw_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<S> AsSocket for ProxySocket<S>
+where
+    S: AsSocket,
+{
+    fn as_socket(&self) -> BorrowedSocket<'_> {
+        self.io.as_socket()
+    }
+}
+
+#[cfg(windows)]
+impl<S> IntoRawSocket for ProxySocket<S>
+where
+    S: IntoRawSocket,
+{
+    fn into_raw_socket(self) -> RawSocket {
+        self.io.into_raw_socket()
     }
 }
